package org.example.country.size.action;

import org.example.country.size.util.ServiceException;

/**
 * This is the action interface for the Text Writer access class.
 * <br>
 * <br>
 * This defines the access methods for output file writer.
 * 
 * @author Jonathan Earl
 *
 */
public interface TextWriter
{
	/**
	 * This lists the output header types.
	 *
	 */
	public enum HEADER {LARGEST, SMALLEST};
	
	/**
	 * This is the method used to open the output file.
	 * 
	 * @param fileName the file name to write to
	 * @throws ServiceException the Exception thrown if there is file error
	 */
	public void open(String fileName) 
			throws ServiceException;
	
	/**
	 * This is the method to output header lines.
	 * 
	 * @param current the header type to output
	 * @throws ServiceException the Exception thrown if there is file error
	 */
	public void writeHeaderLine(HEADER current) 
			throws ServiceException;
	
	/**
	 * This is the method to write a String to the output file.
	 * 
	 * @param line the String to output
	 * @throws ServiceException the Exception thrown if there is file error
	 */
    public void writeLine(String line) 
    		throws ServiceException; 
    
    /**
     * This method outputs a blank line.
     * 
     * @throws ServiceException the Exception thrown if there is file error
     */
    public void writeSpacer()
    		throws ServiceException;
    
    /**
     * This will close the output file.
     * 
     * @throws ServiceException the Exception thrown if there is file error
     */
    public void close() 
    		throws ServiceException;
}
