package org.example.country.size.action;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.commons.lang.StringUtils;
import org.example.country.size.util.ServiceException;

/**
 * This is the implementation of the TextWriter interface.
 * 
 * @author Jonathan Earl
 * @see TextWriter
 *
 */
public class TextWriterImpl
    implements TextWriter
{
    private BufferedWriter writer = null;

    /**
     * {@inheritDoc}
     */
    @Override
    public void open(String fileName)
    	throws ServiceException
    {
        try
		{
			writer = new BufferedWriter(new FileWriter(fileName));
		} 
        catch (Exception e) 
		{
        	throw new ServiceException(e);
		}
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void close() 
    		throws ServiceException
    {
        try
		{
			writer.close();
		} 
        catch (IOException e)
		{
        	throw new ServiceException(e);
		}
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void writeLine(String line) 
            throws ServiceException
    {
        try
		{
			writer.write(line + "\n");
		} 
        catch (IOException e)
		{
        	throw new ServiceException(e);
		}
    }

    /**
     * {@inheritDoc}
     */
	@Override
	public void writeHeaderLine(HEADER current) 
			throws ServiceException
	{
		String tag = null;
		String space = "";
		if (current == HEADER.LARGEST)
		{
			tag = "Largest Countries:";
		}
		else
		{
			tag = "Smallest Countries:";
		}
		writeLine(String.format("%10s %-20s", space, tag));
		
		tag = " %-21s    %-30s        %-10s  %-11s     %-8s";
		writeLine(String.format(tag,  
				"CAPITAL", "NAME",  
				"POPULATION", "SURFACE AREA",
				"DENSITY"));
		tag = " %23s  %32s  %14s  %12s %11s";
		writeLine(String.format(tag,
				StringUtils.repeat("-", 23),
				StringUtils.repeat("-", 32),
				StringUtils.repeat("-", 14),
				StringUtils.repeat("-", 12),
				StringUtils.repeat("-", 10)));		
	}

	/**
     * {@inheritDoc}
     */
	@Override
	public void writeSpacer() throws ServiceException
	{
		writeLine(" ");
	}
}
