package org.example.country.size.action;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.List;
import java.util.Properties;

import org.example.country.size.entity.CountrySize;
import org.example.country.size.test.utilities.DBUnitJDBCUtility;
import org.example.country.size.util.ServiceException;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class CountrySizeDAOTest
{
	
	private static DBUnitJDBCUtility testDatabase = null;
	private static final String COUNTRY_SIZE_FILE = "org/example/country/size/data/country_size.xml";
	private static final String COUNTRY_SIZE_DDL = "org/example/country/size/ddl/CountrySize.sql";

	@BeforeClass
	public static void startUp()
	{
		try
		{
			testDatabase = new DBUnitJDBCUtility(COUNTRY_SIZE_DDL, COUNTRY_SIZE_FILE);
		}
		catch (Exception e)
		{
			fail(e.getLocalizedMessage());
		}
	}  

	@AfterClass
	public static void shutdown() throws Exception
	{
		testDatabase.shutdown();
	}

	@Before
	public void connect() 
			throws Exception
	{
		testDatabase.clean();
	}


	@Test
	public void testFindCountriesBySize() 
			throws Exception
	{
		CountrySizeDAO handler = new CountrySizeDAOImpl();
		handler.connect(getConnectionDetails());
		
		List<CountrySize> results = null;
		results = handler.findCountriesBySize();
		
		int expected = 232;
		int actual = results.size();
		assertEquals(expected, actual);
	}
	
	@Test(expected=ServiceException.class)
	public void testInvalidDriver() 
			throws Exception
	{
		CountrySizeDAO handler = new CountrySizeDAOImpl();
		Properties props = getConnectionDetails();
		props.setProperty("db2.driver", "dummy");
		handler.connect(props);
	}
	
	@Test(expected=ServiceException.class)
	public void testInvalidURL() 
			throws Exception
	{
		CountrySizeDAO handler = new CountrySizeDAOImpl();
		Properties props = getConnectionDetails();
		props.setProperty("db2.url", "dummy");
		handler.connect(props);
	}
	
	@Test
	public void testInvalidSQL() 
			throws Exception
	{
		CountrySizeDAO handler = new CountrySizeDAOImpl();
		handler.connect(getConnectionDetails());
		
		testDatabase.executeSingleDDL("RENAME COLUMN City.Name TO Namex");
		try
		{
			handler.findCountriesBySize();
			fail("Invalid SQL accepted");
		} 
		catch (ServiceException e)
		{
			 Throwable oops = e.getCause();
			 assertTrue(oops instanceof java.sql.SQLSyntaxErrorException);			 
		}
		testDatabase.executeSingleDDL("RENAME COLUMN City.Namex TO Name");
	}
	
	@Test
	public void testDisconnect() 
			throws Exception
	{
		CountrySizeDAO handler = new CountrySizeDAOImpl();
		handler.connect(getConnectionDetails());
		handler.disconnect();
	}
	
	private Properties getConnectionDetails()
	{
		String url = "jdbc:derby:memory:testdb;create=true";
        String userid = "jearl";
        String password = "";
        String driver = "org.apache.derby.jdbc.ClientDriver";
        
        Properties props = new Properties();
        props.setProperty("db2.driver", driver);
        props.setProperty("db2.url", url);
        props.setProperty("db2.user", userid);
        props.setProperty("db2.password", password);
        
        return props;
	}

}
