package org.example.country.size.action;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.example.country.size.util.ServiceException;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;

public class TextWriterTest
{
    @Rule
    public TemporaryFolder folder = new TemporaryFolder();
    
    private File createdFile = null;
    
    private TextWriter fixture = null;
    private String testDataFile = null;

    @BeforeClass  // runs ONCE before any test
    public static void setUpBeforeClass() throws Exception
    {
        // used to ensure all the logging statements run
        Logger  logger = Logger.getLogger(TextWriterImpl.class);
        logger.setLevel(Level.TRACE);
    }

    @Before // runs before EVERY test
    public void setUp() throws Exception
    {
        createdFile = folder.newFile("test.txt");
        fixture = new TextWriterImpl();
        testDataFile = createdFile.getCanonicalPath();
        fixture.open(testDataFile);
    }
    
    @Test
    public void testTextWriterImpl()
    {
        assertNotNull(fixture);
    }
    
    @Test(expected=ServiceException.class)
    public void testInvalidOpen() 
    		throws ServiceException
    {
    	fixture.open(null);
    }
    
    @Test
    public void testWriteHeaderLarger() 
    		throws ServiceException, IOException
    {
    	fixture.writeHeaderLine(TextWriter.HEADER.LARGEST);
    	fixture.close();
    	List<String> linesInFile = readTestFile(testDataFile);

    	int expected = 3;
    	int actual = linesInFile.size();
    	assertEquals(expected, actual);
    	
    	String expectedMessage = "Largest Countries:";
    	String actualMessage = linesInFile.get(0);
    	assertTrue(actualMessage.contains(expectedMessage));
    }
    
    @Test
    public void testWriteHeaderSmaller() 
    		throws ServiceException, IOException
    {
    	fixture.writeHeaderLine(TextWriter.HEADER.SMALLEST);
    	fixture.close();
    	List<String> linesInFile = readTestFile(testDataFile);

    	int expected = 3;
    	int actual = linesInFile.size();
    	assertEquals(expected, actual);
    	
    	String expectedMessage = "Smallest Countries:";
    	String actualMessage = linesInFile.get(0);
    	assertTrue(actualMessage.contains(expectedMessage));
    }
    
    @Test
    public void testWriteLine() 
    		throws ServiceException, IOException
    {
    	String sampleLine = "ABCabc123  ";
    	fixture.writeLine(sampleLine);
    	fixture.writeLine(sampleLine);
    	fixture.writeLine(sampleLine);
    	fixture.close();
    	List<String> linesInFile = readTestFile(testDataFile);

    	int expected = 3;
    	int actual = linesInFile.size();
    	assertEquals(expected, actual);
    	
    	for (String entry : linesInFile)
    	{
    		assertEquals(sampleLine, entry);
    	}
    }
    
    private List<String> readTestFile(String fileName) 
    		throws IOException
    {
    	BufferedReader reader = 
                new BufferedReader(new FileReader(testDataFile));
        
        List<String> data = new ArrayList<String>();
        
        while (true)
        {
            String line  = reader.readLine();
            if (line == null)
            {
                break;
            }
            data.add(line);
        }
        reader.close();
        return data;
    }

   
}
