package org.example.country.size.entity;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.example.country.size.util.ServiceException;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class CountrySizeTest
{
	private CountrySize countrySizeTestObject;

	@BeforeClass   // this run ONCE before ANY test
	public static void setUpBeforeClass() throws Exception
	{
		 Logger  logger = Logger.getLogger(CountrySize.class);
	     logger.setLevel(Level.TRACE);
	}

	@Before   // this run BEFORE EACH test
	public void setUp() throws Exception
	{
		countrySizeTestObject = new CountrySize();
	}
	
	@Test
	public void testInitialCapital()
	{
		String expected = "Birnin Zana";
		String actual = countrySizeTestObject.getCapital();
		assertEquals(expected, actual);
	}

	@Test(expected=ServiceException.class)
	public void testNullCapitol() 
			throws ServiceException
	{
		countrySizeTestObject.setCapital(null);
	}
	
	@Test(expected=ServiceException.class)
	public void testEmptyCapitol() 
			throws ServiceException
	{
		countrySizeTestObject.setCapital("  ");
	}
	
	@Test(expected=ServiceException.class)
	public void testTooLongCapitol() 
			throws ServiceException
	{
		countrySizeTestObject.setCapital("123456789012345678901234567890123456");
	}
	
	@Test
	public void testValidCapitol() 
			throws ServiceException
	{
		countrySizeTestObject.setCapital("Rome");
	}
	
	@Test
	public void testInitialName()
	{
		String expected = "Wakanda";
		String actual = countrySizeTestObject.getName();
		assertEquals(expected, actual);
	}
	
	@Test(expected=ServiceException.class)
	public void testNullName() 
			throws ServiceException
	{
		countrySizeTestObject.setName(null);
	}
	
	@Test(expected=ServiceException.class)
	public void testEmptyName() 
			throws ServiceException
	{
		countrySizeTestObject.setName("  ");
	}
	
	@Test
	public void testValidName() 
			throws ServiceException
	{
		countrySizeTestObject.setName("Italy");
	}
	
	@Test
	public void testInitialPopulation()
	{
		int expected = 6000000;
		int actual = countrySizeTestObject.getPopulation();
		assertEquals(expected, actual);
	}
	
	@Test(expected=ServiceException.class)
	public void testZeroPopulaiton()
			throws ServiceException
	{
		countrySizeTestObject.setPopulation(0);
	}
	
	@Test
	public void testValidPopulation() 
			throws ServiceException
	{
		countrySizeTestObject.setPopulation(120345900);
	}
	
	@Test
	public void testInitialArea()
	{
		double fuzzValue = 0.001;
		double expected = 22384.0;
		double actual = countrySizeTestObject.getArea();
		assertEquals(expected, actual, fuzzValue);
	}

	@Test(expected=ServiceException.class)
	public void testZeroArea()
			throws ServiceException
	{
		countrySizeTestObject.setArea(0);
	}
	
	@Test
	public void testValidArea() 
			throws ServiceException
	{
		countrySizeTestObject.setArea(2321312.23);
	}
	
	@Test
	public void testInitialDensity()
	{
		double fuzzValue = 0.001;
		double expected = 268.049; // actual value is: 268.04860614724805
		double actual = countrySizeTestObject.getDensity();
		assertEquals(expected, actual, fuzzValue);
	}
	
	@Test(expected=ServiceException.class)
	public void testZeroDensity()
			throws ServiceException
	{
		countrySizeTestObject.setDensity(0);
	}
	
	@Test
	public void testValidDensity() 
			throws ServiceException
	{
		countrySizeTestObject.setDensity(12.34);
	}
	
	@Test
	public void testFormat() 
			throws ServiceException
	{
		String expected = " Birnin Zana              "
				+ "Wakanda                                "
				+"6,000,000      22,384.0      268.05";
		String actual = countrySizeTestObject.format();
		assertEquals(expected, actual);
	}
	
	@Test
	public void testToString() 
			throws ServiceException
	{
		String expected = "CountrySize [capital=Birnin Zana, name=Wakanda, "
				+ "population=6000000, area=22384.0, density=268.04860614724805]";
		String actual = countrySizeTestObject.toString();
		assertEquals(expected, actual);
	}
	
	@Test public void testHashCode() 
			throws ServiceException
	{
	    CountrySize sample = new CountrySize();
	    int expected = sample.hashCode();
	    int actual = countrySizeTestObject.hashCode();
	    assertEquals(expected, actual);
	}
	
	@Test public void testEquals() 
			throws ServiceException
    {
		CountrySize sample = new CountrySize();
        assertTrue(countrySizeTestObject.equals(sample));
    }
	
	@Test public void testNotEquals()
			throws ServiceException
    {
		CountrySize sample = new CountrySize();
        sample.setCapital("Smithsville");
        assertFalse(countrySizeTestObject.equals(sample));
    }
	
	@Test public void testNotEqualsType()
			throws ServiceException
    {		
        Object junk = new Object();
        assertFalse(countrySizeTestObject.equals(junk));
    }
	
}
