package org.example.country.size.test.utilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Properties;

import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.datatype.DefaultDataTypeFactory;
import org.dbunit.dataset.xml.FlatDtdDataSet;
import org.dbunit.dataset.xml.FlatXmlWriter;
import org.dbunit.ext.db2.Db2Connection;


public class ExtractDataFromDatabase
{
	private static String exportDir = "test/org/example/country/size/data";
	
	private static final String DTD_NAME = "db.dtd";
	private static final String DTD_FILE = exportDir + File.separator + DTD_NAME;
	private static final String COUNTRY_SIZE_FILE = exportDir + File.separator + "country_size.xml";
	
	public static void main(String[] args) throws Exception
	{
		//  Read the properties file
		Properties prop = new Properties();
		InputStream input = new FileInputStream("CountrySize.properties");

		prop.load(input);
		
		String driver = prop.getProperty("db2.driver");
		String url = prop.getProperty("db2.url");
		String user = prop.getProperty("db2.user");
		String password = prop.getProperty("db2.password");
		
		Class.forName(driver);
		// database connection
		
		Connection jdbcConnection = DriverManager.getConnection(url, user, password);
		IDatabaseConnection connection = new Db2Connection(jdbcConnection, "JEARL");
		DatabaseConfig config = connection.getConfig(); 
		config.setProperty(DatabaseConfig.FEATURE_DATATYPE_WARNING, false);  
        config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new DefaultDataTypeFactory());                  
      
        // write DTD file
        FlatDtdDataSet.write(connection.createDataSet(), new FileOutputStream(DTD_FILE));

		// country size database export
        FlatXmlWriter countrySizeWriter = new FlatXmlWriter(new FileOutputStream(COUNTRY_SIZE_FILE));
        countrySizeWriter.setDocType(DTD_NAME); 
		QueryDataSet countrySizeDataSet = new QueryDataSet(connection);	
		countrySizeDataSet.addTable("jearl.cities");
		countrySizeDataSet.addTable("jearl.countries");
		countrySizeWriter.write(countrySizeDataSet);

	}
}
